/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <stdint.h>

#include <QString>

class Field
{
public:
    enum class Type
    {
        INT,
        UINT,
        DOUBLE,
        STRING,
        EMPTY
    };

    Field();
    explicit Field(int64_t value);
    explicit Field(uint64_t value);
    explicit Field(double value);
    explicit Field(QString value);

    bool operator==(const Field& toCompare) const;

    Type getType() const;

    int64_t getInteger() const;

    uint64_t getUnsignedInteger() const;

    double getDouble() const;

    QString toString() const;

private:
    Type type;
    QString string;

    union
    {
        int64_t signedInteger;
        uint64_t unsignedInteger;
        double floatingPoint;
    };
};