/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Components/ToggleButton/ToggleButton.h>
#include <Core/Message/Message.h>
#include <Modules/Module.h>

#include <QTimer>
#include <QWidget>

#include "MavlinkCodec.h"
#include "Ports/MavlinkPort.h"

Q_DECLARE_METATYPE(Message);

namespace Ui
{
class BaseMavlinkModule;
}

class BaseMavlinkModule : public Module
{
    Q_OBJECT
public:
    BaseMavlinkModule(MavlinkPort *port, ModuleId id);
    ~BaseMavlinkModule();

    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject &obj) override;

protected:
    virtual void childDisableControls()                   = 0;
    virtual void childEnableControls()                    = 0;
    virtual XmlObject childToXmlObject()                  = 0;
    virtual void childFromXmlObject(const XmlObject &obj) = 0;
    virtual bool open()                                   = 0;

    QHBoxLayout *childLayout;
private slots:
    void onMsgReceived(const Message &msg);
    void onLinkQualityTimerTick();
    void onStartStreamToggled(bool state);

private:
    void onStartClicked();
    void onStopClicked();
    void onCommandReceived(const Message &msg);
    void onOpenLogFolderClick();

private:
    void enableControls();
    void disableControls();

    void stop();

    void setupUi();

    MavlinkPort *port;
    MavlinkCodec mavlinkCodec;

    ToggleButton *startToggleButton;
    QComboBox *sysIdComboBox;

    QLabel *rateLabel;
    QCheckBox *logCheckBox;

    QTimer linkQualityTimer;
    const int linkQualityPeriod = 2000;  // [ms]
    int msgArrived              = 0;

    static int ACTIVE_MAVLINK_MODULES;
};