/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Modules/Module.h>
#include <Qt3DRender/qpointlight.h>

#include <QMesh>
#include <QSizePolicy>
#include <QSortPolicy>
#include <QVector3D>
#include <Qt3DCore/QAspectEngine>
#include <Qt3DCore/QEntity>
#include <Qt3DCore/QTransform>
#include <Qt3DExtras/QConeMesh>
#include <Qt3DExtras/QCuboidMesh>
#include <Qt3DExtras/QCylinderMesh>
#include <Qt3DExtras/QFirstPersonCameraController>
#include <Qt3DExtras/QForwardRenderer>
#include <Qt3DExtras/QGoochMaterial>
#include <Qt3DExtras/QOrbitCameraController>
#include <Qt3DExtras/QPhongMaterial>
#include <Qt3DExtras/QSphereMesh>
#include <Qt3DExtras/QTorusMesh>
#include <Qt3DExtras/Qt3DWindow>
#include <Qt3DInput/QInputAspect>
#include <Qt3DRender/QAttribute>
#include <Qt3DRender/QBuffer>
#include <Qt3DRender/QCamera>
#include <Qt3DRender/QCameraLens>
#include <Qt3DRender/QGeometry>
#include <Qt3DRender/QGeometryRenderer>
#include <Qt3DRender/QRenderAspect>

class OrientationVisualizer : public Module
{
    Q_OBJECT

public:
    OrientationVisualizer();

protected:
    void resizeEvent(QResizeEvent *event) override;

private:
    ~OrientationVisualizer() override;

    QLabel *rollLabel, *pitchLabel, *yawLabel;
    QComboBox *sourceComboBox;
    QLabel *sourceLabel;

    Qt3DExtras::Qt3DWindow *view;
    Qt3DCore::QEntity *root;
    Qt3DCore::QTransform *rocketTransform;
    Qt3DRender::QCamera *camera;

    const float PROJECTION_SIZE = 30.;
    const int GRID_SPAN         = 15;
    const float GRID_GAP        = 4.f;

    /**
     * Updates the model orientation. Expects a quaternion from the NED
     * reference system to the body one.
     * @param q NED to body quaternion.
     */
    void updateOrientation(QQuaternion q);
    void setupUi();

    void setupScene();
    void drawAxes();
    void drawAxis(QVector3D dir, const QColor &color);
    void addLight();
    void addCamera();
    void addRocket();
    void addGrid();
    void addRotationLabels(QHBoxLayout *container);

    static QQuaternion rotationFromDirections(QVector3D start, QVector3D end);
    static QVector3D NedToStdVector(float x, float y, float z);
    static QVector3D stdToNedVector(float x, float y, float z);
};
